<?php
namespace Teknisa\FAI\Modules\Apuracao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Fai00900EmissaoNfsApuradas extends \Zeedhi\Framework\Controller\Report {

    protected $qr2Strategy;
    protected $birtStrategy;
    protected $environment;
    protected $newCode;
    protected $procedureApuracRelgesdireta;


    public function __construct($reportService, $qr2Strategy, $birtStrategy, $environment, $newCode, StoredProcedure $procedureApuracRelgesdireta) {
        $this->qr2Strategy                  = $qr2Strategy;
        $this->birtStrategy                 = $birtStrategy;
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
        $this->procedureApuracRelgesdireta  = $procedureApuracRelgesdireta;
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'FAI00900' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                "NRORG"     => "P_NRORG",
                'CDFILIAL'  => 'P_FILIAL',
                'INICIAL'   => 'P_INICIAL',
                'FINAL'     => 'P_FINAL',
                'CDCLIENTE' => 'P_CLIENTE'
            )
        ),

        'FAI00901' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'  => 'P_FILIAL',
                'INICIAL'   => 'P_INICIAL',
                'FINAL'     => 'P_FINAL',
                'CDCLIENTE' => 'P_CLIENTE',
                "NRORG"     => "P_NRORG"
            )
        ),

        'FAI00902' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'   => 'P_FILIAL',
                'INICIAL'    => 'P_INICIAL',
                'FINAL'      => 'P_FINAL',
                'CDCLIENTE'  => 'P_CLIENTE',
                "NRORG"      => "P_NRORG"
            )
        ),
//Todo - Transformar o cds do Delphi em procedure para utilizar o relatorio no Mobile.
        // 'FAI00903' =>
        // array (
        //     'strategy' => 'QR2',
        //     'parameterMapping' =>
        //     array (
        //         'CDFILIAL'  => 'P_FILIAL',
        //         'INICIAL'   => 'P_INICIAL',
        //         'FINAL'     => 'P_FINAL',
        //         'CDCLIENTE' => 'P_CLIENTE'
        //     )
        // ),

        'FAI00904' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_FILIAL',
                'INICIAL'     => 'P_INICIAL',
                'FINAL'       => 'P_FINAL',
                'CDCLIENTE'   => 'P_CLIENTE',
                'NRORG'       => 'P_NRORG'
            )
        ),

        'FAI00905' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'  => 'P_FILIAL',
                'INICIAL'   => 'P_INICIAL',
                'FINAL'     => 'P_FINAL',
                'CDCLIENTE' => 'P_CLIENTE',
                'NRORG'     => 'P_NRORG'
            )
        ),

        'FAI00906' =>
        array (
            'strategy' => 'QR2',
            'parameterMapping' =>
            array (
                'CDFILIAL'  => 'P_FILIAL',
                'INICIAL'   => 'P_INICIAL',
                'FINAL'     => 'P_FINAL',
                'CDCLIENTE' => 'P_CLIENTE'
            )
        ),

        'FAI00907' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_FILIAL',
                'INICIAL'     => 'P_INICIAL',
                'FINAL'       => 'P_FINAL',
                'CDCLIENTE'   => 'P_CLIENTE',
                'NRAPURACFAT' => 'P_NRAPURACGD',
                'NRORG'       => 'P_NRORG'
            )
        ),

        'FAI00908' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'  => 'P_FILIAL',
                'INICIAL'   => 'P_INICIAL',
                'FINAL'     => 'P_FINAL',
                'CDCLIENTE' => 'P_CLIENTE',
                'NRORG'     => 'P_NRORG'
            )
        ),
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $row['NRORG'] = $this->environment->getNrOrgTrab();

        $cdfilial  = $row['CDFILIAL'];
        $dataini   = $row['INICIAL'];
        $datafin   = $row['FINAL'];

        if($row['TCDCLIENTE'] === 'T'){
            $cdcliente = NULL;
            $row['CDCLIENTE'] = NULL;
        }else {
            $cdcliente = $row['CDCLIENTE'];
        }

        if($row["__report_name"] === "FAI00907" || $row["__report_name"] === "FAI00906"){
            $nrapuracfat = $this->newCode->getCodeByTableName('NRAPURACFAT', $this->environment->getNrOrgTrab());
            $row['NRAPURACFAT'] = $nrapuracfat;

            $this->procedureApuracRelgesdireta->addParam(new Param('P_FILIAL',      Param::PARAM_INPUT, $cdfilial));
            $this->procedureApuracRelgesdireta->addParam(new Param('P_INICIAL',     Param::PARAM_INPUT, $dataini));
            $this->procedureApuracRelgesdireta->addParam(new Param('P_FINAL',       Param::PARAM_INPUT, $datafin));
            $this->procedureApuracRelgesdireta->addParam(new Param('P_NRAPURACFAT', Param::PARAM_INPUT, $nrapuracfat));
            $this->procedureApuracRelgesdireta->addParam(new Param('P_CLIENTE',     Param::PARAM_INPUT, $cdcliente));

            if($row["__report_name"] === "FAI00907"){
                $this->procedureApuracRelgesdireta->execute();
            }
        }
    }
}