<?php
namespace Teknisa\FAI\Modules\Apuracao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Fai04700ControleDeCompensacao extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $crudService;
	protected $parameterBag;
	protected $environment;
	protected $dataSourceManager;
	protected $novoCodigo;


	public function __construct($crudService, $parameterBag, $environment, $dataSourceManager, $novoCodigo, $entityManager) {

		$this->crudService			= $crudService;
		$this->parameterBag    		= $parameterBag;
		$this->environment      	= $environment;
		$this->dataSourceManager	= $dataSourceManager;
		$this->novoCodigo			= $novoCodigo;
		$this->entityManager        = $entityManager;
	}

	public function gerarNF(DTO\Request\DataSet $request, DTO\Response $response){
        try {
			$rows = $request->getDataSet()->getRows();
        	$connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            if ($this->existeNota($rows[0]['CDFILIAL'], $rows[0]['NRAPURACCOMP'])) {
				if ($this->validaStatusNF($rows[0]['CDFILIAL'], $rows[0]['NRAPURACCOMP'])) {
	            	$this->geraNotaFiscalProcedure($rows);

				} else {
					throw new \Exception("docsNaoEmitidos", 1);
				}
            } else {
            	throw new \Exception("notaNaoExiste", 1);
            }

            $connection->commit();
        } catch (\Exception $e){
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    private function geraNotaFiscalProcedure($rows) {
    	$servicos = "_";
    	foreach ($rows as $row) {
    		$servicos .= $row["CDSERVFILI"] . "_";
    	}
		$procedure = new StoredProcedure($this->entityManager->getConnection(), 'APURAC_GESDIRETA');
		$procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $rows[0]["CDFILIAL"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $rows[0]["CDCLIENTE"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $rows[0]["DTBASE"]));
		$procedure->addParam(new Param('P_DATAFIM', Param::PARAM_INPUT, $rows[0]["DTBASE"]));
		$procedure->addParam(new Param('P_NRAPURACFAT', Param::PARAM_INPUT, $rows[0]["NRAPURACCOMP"]));
		$procedure->addParam(new Param('P_IDGERANOTA', Param::PARAM_INPUT, 'S'));
		$procedure->addParam(new Param('P_DTSAIDAPUR', Param::PARAM_INPUT, $rows[0]["DTBASE"]));
		$procedure->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT, null));
		$procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $servicos));
		$procedure->execute();
    }

    private function validaStatusNF($cdfilial, $nrapuraccomp) {
		$conditions = array (
			array (
				"columnName"    => "CDFILIAL",
				"operator"      => "=",
				"value"         => $cdfilial
			),
			array (
				"columnName"    => "NRAPURACFAT",
				"operator"      => "=",
				"value"         => $nrapuraccomp
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('fai04700_validastatusnf', $conditions);
		return empty($dataSet->getRows());
	}

	private function existeNota($cdfilial, $nrapuraccomp) {
		$conditions = array (
			array (
				"columnName"    => "CDFILIAL",
				"operator"      => "=",
				"value"         => $cdfilial
			),
			array (
				"columnName"    => "NRAPURACFAT",
				"operator"      => "=",
				"value"         => $nrapuraccomp
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet('fai04700_qtd_nfs_apuradas', $conditions);
		return intval($dataSet->getRows()[0]['QTDE_NF']) > 0;
	}

	private function getNrlanctoNF ($nrapuracfat, $cdcliente) {

		$conditions = array (
			array (
				"columnName" 	=> 'CDFILIAL',
				"operator" 		=> '=',
				"value" 		=> $this->environment->getCdFilial()
			),
			array (
          	  "columnName" 		=> 'NRORG',
          	  "operator" 		=> "=",
          	  "value" 			=> $this->environment->getNrOrg()
          	),
			array (
				"columnName" 	=> 'NRAPURACFAT',
				"operator" 		=> '=',
				"value" 		=> $nrapuracfat
			),
			array (
				"columnName" 	=> 'CDCLIENTE',
				"operator" 		=> '=',
				"value" 		=> $cdcliente
			)
		);
		$filterCriteria = new FilterCriteria('notafiscal', $conditions);
       	$filterCriteria->setDataSourceName('notafiscal');
       	$dataSet = $this->dataSourceManager->findBy($filterCriteria);

		return $dataSet->getRows();
	}

}