<?php
namespace Teknisa\FAI\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Fai40000EstatisticaFaturamento extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName;
    protected $parameterBagColumns = array();
    protected $dataSourceManager;
    protected $tiposDeContrato = array(
        "001" => "GESTÃO DIRETA",
        "002" => "GESTÃO MISTA",
        "003" => "MANDATO",
        "004" => "REFEIÇÕES E SERVIÇOS",
        "005" => "TAXA FIXA",
        "006" => "GESTÃO HOSPITALAR",
        "007" => "PRODUTOS E SERVIÇOS",
        "008" => "GESTÃO LASER",
        "009" => "GESTÃO MERENDA"
    );

    public function __construct($crudService, $parameterBag, $dataSourceManager) {
        $this->crudService  = $crudService;
        $this->parameterBag  = $parameterBag;
        $this->dataSourceManager  = $dataSourceManager;
    }

    public function retornaTipoContrato(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fai40000_tipo_contrato";
            $dataSet = $this->find($request, $response);
            $rows = $this->criaNomeTipoContrato($dataSet->getRows());
            $this->addNewDataSet($rows, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function retornaSerie(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "serie";
            $dataSet = $this->find($request, $response);
            $rows = $this->criaChaveSerie($dataSet->getRows());
            $this->dataSourceName = "fai40000_serie";
            $this->addNewDataSet($rows, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->setDataSourceName($this->dataSourceName);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, $this->parameterBagColumns);
        return $this->dataSourceManager->findBy($newFilterCriteria);
    }

    private function addNewDataSet($rows, $response) {
        $dataSet = new DataSet($this->dataSourceName, $rows);
        $response->addDataSet($dataSet);
    }

    private function criaNomeTipoContrato($rows) {
        foreach ($rows as &$row) {
            $row['NMTIPOFATUR'] = $this->tiposDeContrato[$row['CDTIPOFATUR']];
        }
        return $rows;
    }

    private function criaChaveSerie($rows) {
        foreach ($rows as &$row) {
            $row['CHAVE'] = $row['CDEMPRESA'].$row['CDINSCESTA'].$row['CDINSCMUNI'].$row['CDSERIE'];
        }
        return $rows;
    }
}